/* $Id: init.c,v 1.45 2002/10/26 13:53:24 mikpe Exp $
 * Performance-monitoring counters driver.
 * Top-level initialisation code.
 *
 * Copyright (C) 1999-2002  Mikael Pettersson
 */
#include <linux/config.h>
#include <linux/fs.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/perfctr.h>

#include <asm/uaccess.h>

#include "compat.h"
#include "virtual.h"

#define VERSION "3.0-pre3"

struct perfctr_info perfctr_info = {
	.version = VERSION
};

static inline int sys_perfctr_info(struct perfctr_info *argp)
{
	if( copy_to_user(argp, &perfctr_info, sizeof perfctr_info) )
		return -EFAULT;
	return 0;
}

static int init_done;

asmlinkage int sys_perfctr(unsigned int cmd, int whom, void *arg)
{
	if( !init_done )
		return -ENODEV;
	if( cmd == PERFCTR_INFO )
		return sys_perfctr_info((struct perfctr_info*)arg);
	if( cmd <= VPERFCTR_CREAT )
		return sys_vperfctr(cmd, whom, arg);
	return -EINVAL;
}

int __init perfctr_init(void)
{
	int err;
	if( (err = perfctr_cpu_init()) != 0 ) {
		printk(KERN_INFO "perfctr: not supported by this processor\n");
		return err;
	}
	if( (err = vperfctr_init()) != 0 )
		return err;
	printk(KERN_INFO "perfctr: driver %s, cpu type %s at %lu kHz\n",
	       perfctr_info.version,
	       perfctr_cpu_name[perfctr_info.cpu_type],
	       perfctr_info.cpu_khz);
	init_done = 1;
	return 0;
}

void __exit perfctr_exit(void)
{
	vperfctr_exit();
	perfctr_cpu_exit();
}

module_init(perfctr_init)
module_exit(perfctr_exit)
